Package: mitch
Title: Multi-Contrast Gene Set Enrichment Analysis
Version: 1.24.0
Authors@R: c(
    person(given = "Mark",
        family = "Ziemann",
        role = c("aut","cre","cph"),
        email = "mark.ziemann@gmail.com",
        comment = c(ORCID="0000-0002-7688-6974")),
    person(given="Antony",
        family="Kaspi",
        role = c("aut","cph"))
    )
Description: mitch is an R package for multi-contrast enrichment analysis. At
    it’s heart, it uses a rank-MANOVA based statistical approach to detect sets
    of genes that exhibit enrichment in the multidimensional space as compared
    to the background. The rank-MANOVA concept dates to work by Cox and Mann 
    (https://doi.org/10.1186/1471-2105-13-S16-S12). mitch is useful for 
    pathway analysis of profiling studies with one, two or more contrasts,
    or in studies with multiple omics profiling, for example proteomic,
    transcriptomic, epigenomic analysis of the same samples. mitch is perfectly
    suited for pathway level differential analysis of scRNA-seq data.
    We have an established routine for pathway enrichment of Infinium Methylation
    Array data (see vignette).
    The main strengths of mitch are that it can import datasets easily from
    many upstream tools and has advanced plotting features to visualise these
    enrichments. 
Depends: R (>= 4.4)
Suggests: stringi, testthat (>= 2.1.0), HGNChelper,
        IlluminaHumanMethylation450kanno.ilmn12.hg19,
        IlluminaHumanMethylationEPICanno.ilm10b4.hg19
Imports: stats, grDevices, graphics, utils, MASS, plyr, reshape2,
        parallel, GGally, grid, gridExtra, knitr, rmarkdown, ggplot2,
        gplots, beeswarm, echarts4r, kableExtra, dplyr, network
License: CC BY-SA 4.0 + file LICENSE
Encoding: UTF-8
URL: https://github.com/markziemann/mitch
BugReports: https://github.com/markziemann/mitch
LazyData: false
RoxygenNote: 7.3.2
biocViews: GeneExpression, GeneSetEnrichment, SingleCell,
        Transcriptomics, Epigenetics, Proteomics,
        DifferentialExpression, Reactome, DNAMethylation,
        MethylationArray, DataImport
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/mitch
git_branch: RELEASE_3_23
git_last_commit: 1235d24
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:53:24 UTC; biocbuild
Author: Mark Ziemann [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7688-6974>),
  Antony Kaspi [aut, cph]
Maintainer: Mark Ziemann <mark.ziemann@gmail.com>
Built: R 4.6.0; ; 2026-04-29 04:50:55 UTC; unix
